# -*- coding: utf-8 -*-

# Authors: Roberto Alsina <roberto.alsina@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Customized Setup Account page for SSO."""

import gettext
import re

from PyQt4 import QtGui

from ubuntu_sso.qt import gui as sso_gui

_ = gettext.gettext

bad = u'<font color="red"> %s </font>'
good = u'<font color="green"> %s </font>'

EMPTY_NAME = _("Please enter your name")
INVALID_EMAIL = _("Please enter a valid email address")
EMAIL_MATCH = _("The email addresses do not match")
PASSWORD_LENGTH = _("At least 8 characters")
PASSWORD_DIGIT = _("At least one number")
PASSWORD_UPPER = _("At least one uppercase letter")
PASSWORD_MATCH = _("Passwords don't match")


class SetupAccountPage(sso_gui.SetupAccountPage):

    """Customized Setup Account page for SSO"""

    def initializePage(self):
        """Setup UI details."""
        # We need to override some texts from SSO
        # to match our spec
        self.setTitle(_("Sign Up to Ubuntu One"))
        self.setSubTitle(_("You only need to set up your account "
        "once to get access to Ubuntu One across your devices."))

        self.ui.name_label.setText(_("Name"))
        self.ui.email_label.setText(_("Email"))
        self.ui.confirm_email_label.setText(_("Retype email"))
        self.ui.password_label.setText(_("Create a password"))
        self.ui.confirm_password_label.setText(_("Retype password"))
        self.ui.terms_checkbox.setText(
            _("By signing up to Ubuntu One you agree to our Terms "
            "of Service and Privacy Policy"))
        self.ui.terms_button.setText(_("Show Terms of Service"))
        self.ui.password_info_label.hide()

        # Button setup
        self.wizard().setButtonLayout([
            QtGui.QWizard.Stretch,
            QtGui.QWizard.BackButton,
            QtGui.QWizard.CancelButton,
            ])

        self.ui.name_edit.textEdited.connect(self.name_assistance)
        self.name_assistance()
        self.ui.email_edit.textEdited.connect(self.email_assistance)
        self.ui.email_edit.textEdited.connect(self.confirm_email_assistance)
        self.email_assistance()
        self.ui.confirm_email_edit.textEdited.connect(
            self.confirm_email_assistance)
        self.confirm_email_assistance()
        self.ui.password_edit.textEdited.connect(self.password_assistance)
        self.ui.confirm_password_edit.textEdited.connect(self.password_assistance)
        self.password_assistance()

    def name_assistance(self):
        text = unicode(self.ui.name_edit.text())
        if not text.strip():
            self.ui.name_assistance.setVisible(True)
            self.ui.name_assistance.setText(bad % EMPTY_NAME)
        else:
            self.ui.name_assistance.setVisible(False)

    def email_assistance(self):
        text = unicode(self.ui.email_edit.text())
        if not is_correct_email(text):
            self.ui.email_assistance.setText(bad % INVALID_EMAIL)
            self.ui.email_assistance.setVisible(True)
        else:
            self.ui.email_assistance.setVisible(False)

    def confirm_email_assistance(self):
        text1 = unicode(self.ui.email_edit.text())
        text2 = unicode(self.ui.confirm_email_edit.text())
        if text1 != text2:
            self.ui.confirm_email_assistance.setText(bad % EMAIL_MATCH)
            self.ui.confirm_email_assistance.setVisible(True)
        else:
            self.ui.confirm_email_assistance.setVisible(False)

    def password_assistance(self):
        text1 = unicode(self.ui.password_edit.text())
        text2 = unicode(self.ui.confirm_password_edit.text())
        label_text = [_("Your password must contain"), ]

        if len(text1) < 8:
            sign = bad
        else:
            sign = good
        label_text.append(sign % PASSWORD_LENGTH)

        if re.search('[A-Z]', text1) is None:
            sign = bad
        else:
            sign = good
        label_text.append(sign % PASSWORD_UPPER)

        if re.search('[\d+]', text1) is None:
            sign = bad
        else:
            sign = good
        label_text.append(sign % PASSWORD_DIGIT)

        if text1 != text2:
            label_text.append(bad % PASSWORD_MATCH)

        self.ui.password_assistance.setText("<br>".join(label_text))


def is_min_required_password(password):
    """Return if the password meets the minimum requirements."""
    if (len(password) < 8 or
        re.search('[A-Z]', password) is None or
        re.search('\d+', password) is None):
        return False
    return True


# FIXME: this should do the same check as SSO's server side
def is_correct_email(email_address):
    """Return if the email is correct."""
    return '@' in email_address
